@@ -1,9 +1,9 @@
-__version__ = "0.3.0"
+__version__ = "1.0.1"
 __doc__ = """GEMAct is an **actuarial package**, 
 based on the collective risk theory framework, 
 that offers actuarial scientists a comprehensive set of tools for 
-**non-life** (re)insurance **pricing**, stochastic **claims reserving**, 
-**risk aggregation** and extends the set of probability distributions already available in Python. \n
+**non-life** (re)insurance **costing**, stochastic **claims reserving**, 
+**risk aggregation** and extends the set of probability distributions available in Python. \n
 The broad and flexible GEMAct apparatus fits into the expanding community of **Python** programming language. \n
 Please visit our [website](https://gem-analytics.github.io/gemact/) to see our documentation and tutorials."""
 
