@@ -2,7 +2,7 @@ from .libraries import *
 from . import config
 from . import helperfunctions as hf
 from . import distributions as distributions
-from .calculators import LossModelCalculator as Calculator 
+from .calculators import LossModelCalculator as Calculator
 
 quick_setup()
 logger = log.name('lossmodel')
@@ -26,7 +26,7 @@ class PolicyStructure:
     def layers(self, value):
         hf.assert_type_value(
             value, 'layers',
-            type=(LayerTower, Layer, list), logger=logger
+            type=(Layer, list), logger=logger
             )
         if isinstance(value, Layer):
             value = [value]
@@ -43,6 +43,9 @@ class PolicyStructure:
         return len(self.layers)
 
     def index_to_layer_name(self, idx):
+        """
+        Return name of a layer given its index.
+        """
         message = 'ValueError.\n No Layer has index %s.' % (idx)
         try:
             output = self.layers[idx].name
@@ -53,6 +56,9 @@ class PolicyStructure:
         return output
     
     def layer_name_to_index(self, name):
+        """
+        Return index of a layer given its name.
+        """
         output = [idx for idx in range(self.length) if self.layers[idx].name == name][0]
         if not output:
             message = 'ValueError. No Layer has name "%s".' % (name)
@@ -328,88 +334,6 @@ class Layer:
         )
         return
 
-class LayerTower(list):
-    """
-    Policy structure tower of non-proportional layers.
-
-    :param \\**args:
-        See below
-
-    :Keyword Arguments:
-        * *args* (``Layers``) --
-          Layer tower elements.
-    """
-
-    def __init__(self, *args):
-        for arg in args:
-            hf.assert_type_value(arg, 'item', logger, Layer)
-        super(LayerTower, self).__init__(args)
-        # self.extend(args)
-        self.set_and_check_tower()
-
-    def append(self, item):
-        hf.assert_type_value(item, 'item', logger, Layer)
-        super(LayerTower, self).append(item)
-    
-    def insert(self, item):
-        hf.assert_type_value(item, 'item', logger, Layer)
-        super(LayerTower, self).insert(item)
-    
-    def extend(self, *args):
-        for arg in args:
-            hf.assert_type_value(arg, 'item', logger, Layer)
-        super(LayerTower, self).extend(args)
-    
-    def sort(self, key='deductible'):
-        hf.assert_member(key, Layer.specs(), logger)
-        super(LayerTower, self).sort(
-            key=lambda x: getattr(x, key)
-            )
-    
-    def remove_layer_loading(self):
-        for elt in self:
-            elt.reinst_loading = 0
-
-    def set_and_check_tower(self):
-        self.remove_duplicates()
-        self.sort()
-        hf.check_condition(
-            value=self[0].deductible,
-            check=0,
-            name='First layer (retention) deductible',
-            logger=logger,
-            type='=='
-            )
-        for i in range(1, len(self)):
-            hf.check_condition(
-                value=self[i].category,
-                check='xlrs',
-                name='category of ' + self[i].name,
-                logger=logger,
-                type='!='
-                )
-            hf.check_condition(
-                value=self[i].deductible,
-                check=self[i-1].deductible + self[i-1].cover,
-                name='deductible of ' + self[i].name,
-                logger=logger,
-                type='=='
-                )
-            if self[i].basis == 'regular':
-                logger.warning('Having regular basis may generate noncontiguous layers.')
-
-    def remove_duplicates(self):
-        memory = []
-        for element in self:
-            if element.identifier not in memory:
-                memory.append(element.identifier)
-            else:
-                self.remove(element)
-                logger.warning(
-                    'Removing %s as a duplicate of another Layer.' %(element.name)
-                    )
-        return
-
 class Frequency:
     """
     Frequency component of the loss models underlying the collective risk model.
@@ -610,7 +534,8 @@ class Severity:
         :type deductible: ``int`` or ``float``
         :param cover: cover, also referred to as limit.
         :type cover: ``int`` or ``float``
-        :param discr_method: severity discretization method. One of 'massdispersal', 'localmoments'.
+        :param discr_method: severity discretization method. One of 'massdispersal', 'localmoments',
+                             'upperdiscretization', 'lowerdiscretization'.
         :type discr_method: ``str``
         :param discr_step: severity discretization step.
         :type discr_step: ``float``
@@ -653,10 +578,98 @@ class Severity:
                 n_discr_nodes
                 )
 
+        elif discr_method == 'lowerdiscretization':
+            return Calculator.lower_discretization(
+                self,
+                deductible,
+                exit_point,
+                discr_step,
+                n_discr_nodes
+                )
+
+        elif discr_method == 'upperdiscretization':
+            return Calculator.upper_discretization(
+                self,
+                deductible,
+                exit_point,
+                discr_step,
+                n_discr_nodes
+                )
+
+    def plot_discr_sev_cdf(
+            self,
+            discr_method,
+            n_discr_nodes,
+            discr_step,
+            cover,
+            deductible,
+            log_x_scale=False,
+            log_y_scale=False,
+            **kwargs
+            ):
+        """
+        Plot the cumulative distribution function of the discretized severity distribution.
+
+        :param discr_method: severity discretization method. One of 'massdispersal', 'localmoments',
+                                'upperdiscretization', 'lowerdiscretization'.
+        :type discr_method: ``str``
+        :param n_discr_nodes: number of nodes of the discretized severity.
+        :type n_discr_nodes: ``int``
+        :param discr_step: severity discretization step.
+        :type discr_step: ``float``
+        :param deductible: deductible, also referred to as retention or priority.
+        :type deductible: ``int`` or ``float``
+        :param cover: cover, also referred to as limit.
+        :type cover: ``int`` or ``float``
+        :param log_x_scale: if ``True`` the x-axis scale is logarithmic (optional).
+        :type log_x_scale: ``bool``
+        :param log_y_scale: if ``True`` the y-axis scale is logarithmic (optional).
+        :type log_y_scale: ``bool``
+
+        :param \\**kwargs:
+            Additional parameters as those for ``matplotlib.axes.Axes.step``.
+
+        :return: plot of the cdf.
+        :rtype: ``matplotlib.figure.Figure``
+        """
+
+        hf.assert_type_value(log_x_scale, 'log_x_scale', logger, bool)
+        hf.assert_type_value(log_y_scale, 'log_y_scale', logger, bool)
+
+        sevdict = self.discretize(
+            discr_method=discr_method,
+            n_discr_nodes=n_discr_nodes,
+            discr_step=discr_step,
+            cover=cover,
+            deductible=deductible
+        )
+
+        x_ = sevdict['nodes']
+        y_ = np.cumsum(sevdict['fj'])
+
+        figure = plt.figure()
+        ax = figure.add_subplot(111)
+
+        ax.step(x_, y_, '-', where='post', **kwargs)
+        if log_y_scale:
+            ax.set_yscale('log')
+        if log_x_scale:
+            ax.set_xscale('log')
+        ax.set_title('Discretized severity cumulative distribution function')
+        ax.set_ylabel('cdf')
+        ax.set_xlabel('nodes')
+        return ax
+
 class LossModel:
     """
     Loss model for (re)insurance costing and risk modeling using a collective risk model framework.
 
+    :param severity: severity model.
+    :type severity: ``Severity``
+    :param frequency: frequency model.
+    :type frequency: ``Frequency``
+    :param policystructure: policy structure.
+    :type policystructure: ``PolicyStructure``
     :param aggr_loss_dist_method: computational method to approximate the aggregate loss distribution.
                                   One of Fast Fourier Transform ('fft'),
                                   Panjer recursion ('recursion') and Monte Carlo simulation ('mc').
@@ -671,7 +684,8 @@ class LossModel:
     :type random_state: ``int``, optional
     :param n_aggr_dist_nodes: number of nodes in the approximated aggregate loss distribution.
     :type n_aggr_dist_nodes: ``int``
-    :param sev_discr_method: severity discretization method. One of 'massdispersal', 'localmoments'.
+    :param sev_discr_method: severity discretization method. One of 'massdispersal', 'localmoments',
+                            'upperdiscretization', 'lowerdiscretization'.
     :type sev_discr_method: ``str``
     :param sev_discr_step: severity discretization step.
     :type sev_discr_step: ``float``
@@ -918,11 +932,8 @@ class LossModel:
                 )
                 if random_state is not None:
                     self.random_state = random_state
-                hf.assert_not_none(
-                    value=self.random_state,
-                    name='random_state',
-                    logger=logger
-                )
+                # Remark: no assert_not_none needed since
+                # self.random_state cannot be None due to hf.handle_random_state
                 
                 logger.info('Approximating aggregate loss distribution via Monte Carlo simulation')
                 aggr_dist_excl_aggr_cond = Calculator.mc_simulation(
@@ -1126,35 +1137,12 @@ class LossModel:
         :return: quantile.
         :rtype: ``numpy.float64`` or ``numpy.ndarray``
         """
-        # hf.assert_type_value(
-        #     q, 'q', logger, (np.floating, int, float, list, np.ndarray)
-        #     )
         hf.assert_type_value(
             idx, 'idx', logger, int,
             upper_bound=len(self.dist)-1,
             lower_bound=0
             )
         self._check_dist(idx)
-        # aggr_loss_dist_ = self.dist[idx]
-
-        # q = np.ravel(q)
-        # for item in q:
-        #     hf.assert_type_value(
-        #         item, 'q', logger,
-        #         (np.floating, np.integer, float, int),
-        #         upper_bound=1, lower_bound=0
-        #         )
-        
-        # probs_ = np.concatenate(
-        #     ([0, 0], aggr_loss_dist_['cdf'], [1, 1])
-        #     )
-        # nodes_ = np.concatenate(
-        #     ([-np.inf, 0],
-        #     aggr_loss_dist_['nodes'],
-        #     [aggr_loss_dist_['nodes'][-1] + config.TOLERANCE, np.inf])
-        #     )
-        # ppf = interp1d(probs_, nodes_)
-        # return ppf(q)  
         return self.dist[idx].ppf(q) 
 
     def cdf(self, x, idx=0):
@@ -1169,28 +1157,34 @@ class LossModel:
         :return: cumulative distribution function.
         :rtype: ``numpy.float64`` or ``numpy.ndarray``
         """
-
         hf.assert_type_value(
             idx, 'idx', logger, int,
             upper_bound=len(self.dist)-1,
             lower_bound=0
             )
         self._check_dist(idx)
-        # aggr_loss_dist_ = self.dist[idx]
-
-        # x = np.ravel(x)
-        # probs_ = np.concatenate(
-        #     ([0, 0], aggr_loss_dist_['cdf'], [1, 1])
-        #     )
-        # nodes_ = np.concatenate(
-        #     ([-np.inf, 0],
-        #     aggr_loss_dist_['nodes'],
-        #     [aggr_loss_dist_['nodes'][-1] + config.TOLERANCE, np.inf])
-        #     )
-        # cdf = interp1d(nodes_, probs_)
-        # return cdf(x)
         return self.dist[idx].cdf(x)
 
+    def sf(self, x, idx=0):
+        """
+        Aggregate loss distribution survival function.
+
+        :param x: quantiles where the survival functionis evaluated.
+        :type x: ``float`` or ``int`` or ``numpy.ndarray``
+        :param idx: list index corresponding to the layer loss distribution of interest (default is 0).
+                    See 'index_to_layer_name' and 'layer_name_to_index' PolicyStructure methods.
+        :type idx: ``int``
+        :return: survival function.
+        :rtype: ``numpy.float64`` or ``numpy.ndarray``
+        """
+        hf.assert_type_value(
+            idx, 'idx', logger, int,
+            upper_bound=len(self.dist)-1,
+            lower_bound=0
+            )
+        self._check_dist(idx)
+        return self.dist[idx].sf(x)
+
     def rvs(self, size=1, random_state=None, idx=0):
         """
         Random variates generator function.
@@ -1212,16 +1206,6 @@ class LossModel:
             lower_bound=0
             )
         self._check_dist(idx)
-        # aggr_loss_dist_ = self.dist[idx]
-
-        # random_state = hf.handle_random_state(random_state, logger)
-        # np.random.seed(random_state)
-
-        # hf.assert_type_value(size, 'size', logger, (float, int), lower_bound=0, lower_close=False)
-        # size = int(size)
-
-        # output = np.random.choice(aggr_loss_dist_['nodes'], size=size, p=aggr_loss_dist_['epmf'])
-        # return output
         return self.dist[idx].rvs(size, random_state)
 
     def mean(self, idx=0):
@@ -1428,7 +1412,7 @@ class LossModel:
         data.extend([['Share partecip.',  layer.share]])
         data.extend([['Pure premium', round(premium, 2)]])
 
-        print('{: >20} {: >25} '.format(' ', *['Costing Summary']))
+        print('{: >20} {: >25} '.format(' ', *['Costing Summary: Layer ' + str(idx)]))
         print('{: >10} {: >35}'.format(' ', *['====================================================================']))
         print('{: >10} {: >35} {: >15} '.format(' ', *['Quantity', 'Value']))
         print('{: >10} {: >35}'.format(' ', *['====================================================================']))
@@ -1454,8 +1438,7 @@ class LossModel:
             )
 
         data = [
-            ['Aggregate loss dist. method', self.aggr_loss_dist_method],
-            ['n_aggr_dist_nodes', self.n_aggr_dist_nodes],
+            ['Aggregate loss dist. method', self.aggr_loss_dist_method]
         ]
 
         if self.aggr_loss_dist_method == 'mc':
@@ -1469,6 +1452,7 @@ class LossModel:
             #     n_sev_to_print += 1
 
             data.extend([
+                ['n_aggr_dist_nodes', self.n_aggr_dist_nodes],
                 ['Sev. discr. method', self.sev_discr_method],
                 ['Sev. discr. step', self.sev_discr_step],
                 ['Number of sev. discr. nodes', n_sev_to_print]
@@ -1540,3 +1524,46 @@ class LossModel:
         hf.assert_not_none(
             value=self.dist[idx], name='dist', logger=logger
         )
+
+    def plot_dist_cdf(self, idx=0, log_x_scale=False, log_y_scale=False, **kwargs):
+        """
+        Plot the cumulative distribution function of the aggregate loss distribution.
+
+        :param idx: index corresponding to the policystructure layer of interest (default is 0).
+                    See 'index_to_layer_name' and 'layer_name_to_index' PolicyStructure methods.
+        :type idx: ``int``
+        :param log_x_scale: if ``True`` the x-axis scale is logarithmic (optional).
+        :type log_x_scale: ``bool``
+        :param log_y_scale: if ``True`` the y-axis scale is logarithmic (optional).
+        :type log_y_scale: ``bool``
+        :param \\**kwargs:
+            Additional parameters as those for ``matplotlib.axes.Axes.step``.
+
+        :return: plot of the cdf.
+        :rtype: ``matplotlib.figure.Figure``
+        """
+
+        hf.assert_type_value(
+            idx, 'idx', logger, int,
+            upper_bound=len(self.dist) - 1,
+            lower_bound=0
+        )
+        self._check_dist(idx)
+        hf.assert_type_value(log_x_scale, 'log_x_scale', logger, bool)
+        hf.assert_type_value(log_y_scale, 'log_y_scale', logger, bool)
+
+        x_ = self.dist[idx].nodes
+        y_ = self.dist[idx].cumprobs
+
+        figure = plt.figure()
+        ax = figure.add_subplot(111)
+
+        ax.step(x_, y_, '-', where='post', **kwargs)
+        if log_y_scale:
+            ax.set_yscale('log')
+        if log_x_scale:
+            ax.set_xscale('log')
+        ax.set_title('Aggregate loss cumulative distribution function')
+        ax.set_ylabel('cdf')
+        ax.set_xlabel('nodes')
+        return ax
