@@ -120,19 +120,16 @@ class AggregateData:
         self.__reported_claims = var
 
     def _ix_setter(self):
-
-        return np.tile(np.arange(0, self.j), self.j).reshape(self.j,
-                                                                  self.j) + np.tile(np.arange(1, self.j + 1),
-                                                                                    self.j).reshape(self.j,
-                                                                                                    self.j).T
+        return np.tile(np.arange(0, self.j), self.j).reshape(self.j, self.j) +\
+                np.tile(np.arange(1, self.j + 1), self.j).reshape(self.j, self.j).T
 
     def _j_setter(self):
-
-        return hf.triangles_dimension(incremental_payments=self.ip_tr,
-                                          cased_payments=self.cp_tr,
-                                          incurred_number=self.in_tr,
-                                          cased_number=self.cn_tr)
-
+        return hf.triangle_dimension(
+            incremental_payments=self.ip_tr,
+            cased_payments=self.cp_tr,
+            incurred_number=self.in_tr,
+            cased_number=self.cn_tr
+            )
 
 class ReservingModel:
     """
@@ -259,7 +256,8 @@ class LossReserve:
                  custom_alphas=None,
                  custom_ss=None,
                  ntr_sim=1000,
-                 random_state=42):
+                 random_state=None
+                 ):
 
         self.data = data
         self.reservingmodel = reservingmodel
@@ -615,7 +613,7 @@ class LossReserve:
                 self.fl_reserve, _, _ = self._fisherlange()
                 return self._stochastic_crm()
 
-    def ss_plot(self, start_=0):
+    def plot_ss_fl(self, start_=0):
         """
         Plot the settlement speed vector for each accident period.
 
@@ -624,25 +622,35 @@ class LossReserve:
         """
 
         x_ = np.arange(0,self.data.j + self.reservingmodel.tail)
-        plt.title('Plot of the settlement speed from accident period %s' % start_)
-        plt.xlabel('Development period')
-        plt.ylabel('Settlement speed')
+
+        figure = plt.figure()
+        ax = figure.add_subplot(111)
+
         for i in range(start_, self.data.j):
-            plt.plot(x_, self.ss_tr[i, :], '-.', label='AP %s' % i)
-            plt.legend()
-        plt.show()
+            ax.plot(x_, self.ss_tr[i, :], '-.', label='AP %s' % i)
+            ax.legend()
 
+        ax.set_title('Plot of the settlement speed from accident period %s' % start_)
+        ax.set_xlabel('Development period')
+        ax.set_ylabel('Settlement speed')
+        return ax
 
-    def alpha_plot(self):
+    def plot_alpha_fl(self):
         """
         Plot the Fisher-Lange alpha.
         """
         x_ = np.arange(0, self.data.j + self.reservingmodel.tail - 1)
-        plt.title('Plot of Alpha')
-        plt.plot(x_, self.alpha_fl, '-.', label='Alpha')
-        plt.xlabel('Development period')
-        plt.ylabel('Alpha')
-        plt.show()
+
+        figure = plt.figure()
+        ax = figure.add_subplot(111)
+
+        ax.plot(x_, self.alpha_fl, '-.', label='Alpha')
+        ax.legend()
+
+        ax.set_title('Plot of Alpha')
+        ax.set_xlabel('Development period')
+        ax.set_ylabel('Alpha')
+        return ax
 
     def _reserve_by_ay_fl(self):
         """
@@ -672,7 +680,7 @@ class LossReserve:
             ultimate = np.round(self.ay_ultimateFL, 2)
             reserve = np.round(self.ay_reserveFL, 2)
 
-        return np.dstack((ay_, ultimate, reserve)).reshape(-1, 3), ['accident period', 'ultimate FL', 'reserve FL']
+        return np.dstack((ay_, ultimate, reserve)).reshape(-1, 3), ['Accident period', 'Ultimate (FL)', 'Reserve (FL)']
 
     def _build_graphic_parameters(self):
         """
@@ -698,7 +706,7 @@ class LossReserve:
             data = np.column_stack((data, data2))
 
             #build header
-            l2_ = ['ultimate CRM', 'reserve CRM', 'm_sep CRM']
+            l2_ = ['Ultimate (CRM)', 'Reserve (CRM)', 'M_sep (CRM)']
             l_.extend(l2_)
             s_ = s_ + " {: >20} {: >20} {: >20}"
 
@@ -711,10 +719,10 @@ class LossReserve:
         """
 
         if self.reservingmodel.model_class == 'average_cost':
-            print('\n FL reserve: ', np.round(self.fl_reserve, 2))
+            print('\n Reserve (FL): ', np.round(self.fl_reserve, 2))
             if self.reservingmodel.reserving_method == 'crm':
-                print('\n CRM reserve: ', np.round(self.reserve, 2))
-                print('\n CRM m_sep: ', np.round(self.m_sep, 2))
+                print('\n Reserve (CRM): ', np.round(self.reserve, 2))
+                print('\n M_sep (CRM): ', np.round(self.m_sep, 2))
 
 
     def print_loss_reserve(self):
